﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports MindFusion.Diagramming.Wpf.Lanes


''' <summary>
''' Interaction logic for MainWindow.xaml
''' </summary>
Partial Public Class MainWindow
	Inherits Window
	Public Sub New()
		InitializeComponent()

        ' Initialize flowchart
        diagram.AutoResize = AutoResize.None
        Dim style As New System.Windows.Style()
        style.Setters.Add(New Setter(DiagramNode.BrushProperty, New SolidColorBrush(Color.FromRgb(&HCF, &HBF, &HEB))))
        diagram.ShapeNodeStyle = style

		pattern = New AnchorPattern(New AnchorPoint() { _
		 New AnchorPoint(100, 50, False, True, MarkStyle.None), _
		 New AnchorPoint(0, 0, True, False, MarkStyle.None)})

		' Initialize grid
		Dim header As Header
        Dim grid As Grid = diagram.LaneGrid

		Dim columns As HeaderCollection = grid.ColumnHeaders

		grid.MinHeaderSize = 30
		'grid.HookHeaders = false;
		grid.HeadersOnTop = False
		grid.ColumnHeadersHeights = New Double() {30, 30}
		grid.AlignCells = False
        grid.AllowResizeHeaders = False

		For i As Integer = 1 To 8
            header = New Header(String.Format("Week {0}, {1}", i, DateTime.Now.Year))
			header.SubHeaders.Add(New Header("S"))
			header.SubHeaders.Add(New Header("M"))
			header.SubHeaders.Add(New Header("T"))
			header.SubHeaders.Add(New Header("W"))
			header.SubHeaders.Add(New Header("T"))
			header.SubHeaders.Add(New Header("F"))
            header.SubHeaders.Add(New Header("S"))
            header.SubHeaders(0).Style.BackgroundBrush = New SolidColorBrush(Color.FromRgb(&HFE, &HF8, &HDA))
            header.SubHeaders(header.SubHeaders.Count - 1).Style.BackgroundBrush = New SolidColorBrush(Color.FromRgb(&HFE, &HF8, &HDA))
			columns.Add(header)
		Next

		For i As Integer = 1 To 6
			grid.RowHeaders.Add(New Header("Task " & i))
			grid.RowHeaders(i - 1).Width = 50
		Next

		Dim count As Integer = 8 * 7
		For i As Integer = 0 To count - 1
			Dim column As Header = grid.GetColumn(i)

			If i Mod 7 <> 0 Then
				grid(column, Nothing).Style.LeftBorderPen = Nothing
			Else
				grid(column, Nothing).Style.LeftBorderPen = New Pen(New LinearGradientBrush(Colors.Gray, Colors.Transparent, 0), 1)
			End If

			If i <> count - 1 Then
				grid(column, Nothing).Style.RightBorderPen = Nothing
			End If

			If i Mod 7 = 0 OrElse i Mod 7 = 6 Then
                grid(column, Nothing).Style.BackgroundBrush = Brushes.WhiteSmoke
			End If
		Next

		count = 6
		For i As Integer = 0 To count - 1
			Dim row As Header = grid.GetRow(i)

			If i = 0 Then
				grid(Nothing, row).Style.TopBorderPen = Nothing
			Else
				grid(Nothing, row).Style.TopBorderPen = New Pen(New LinearGradientBrush(Colors.Gray, Colors.Gray, 0), 0.7)
			End If

			If i <> count - 1 Then
				grid(Nothing, row).Style.BottomBorderPen = Nothing
			End If
		Next

		diagram.EnableLanes = True

		' Ensure the document is big enough to contain the grid
        Dim rect As Rect = grid.GetColumnHeaderBounds()
        rect.Union(grid.GetRowHeaderBounds())
        diagram.Bounds = rect
        diagram.AlignToGrid = False
	End Sub

	Private pattern As AnchorPattern

	''' <summary>
	''' Gets the size of a pixel measured in document units.
	''' </summary>
	Private Function GetPixel() As Double
		Dim pt1 As Point = diagram.ClientToDoc(New Point(0, 0))
		Dim pt2 As Point = diagram.ClientToDoc(New Point(1, 1))

		Return pt2.X - pt1.X
	End Function

	''' <summary>
	''' Fixes the segments of the specified link according
	''' to the position of its related boxes.
	''' </summary>
	Private Shared Sub FixLink(ByVal link As DiagramLink)
		Dim originBounds As Rect = link.Origin.Bounds
		Dim destBounds As Rect = link.Destination.Bounds

		If originBounds.Right < destBounds.Left Then
			link.SegmentCount = 2
		Else
			link.SegmentCount = 4

			Dim pt0 As Point = link.ControlPoints(0)
			Dim pt1 As Point
			Dim pt2 As Point
			Dim pt3 As Point = link.ControlPoints(3)
			Dim pt4 As Point = link.ControlPoints(4)

			pt1 = pt0
			pt1.X += 6

			pt2 = pt1
			pt2.Y += 15

			pt3.X = pt4.X
			pt3.Y = pt2.Y

			link.ControlPoints(1) = pt1
			link.ControlPoints(2) = pt2
			link.ControlPoints(3) = pt3

			link.UpdateFromPoints()
		End If
	End Sub

	Private Sub diagram_HitTestAdjustmentHandles(ByVal sender As Object, ByVal e As HitTestEventArgs)
		If Not (TypeOf e.Item Is ShapeNode) Then
            e.HitResult = Nothing
			Return
		End If

		Dim node = TryCast(e.Item, ShapeNode)

		If Not node.Bounds.Contains(e.MousePosition) Then
            e.HitResult = Nothing
			Return
		End If

		Dim left As Rect = node.Bounds
		left.Width = 2

		If left.Contains(e.MousePosition) Then
            e.HitResult = New NodeAdjustmentHandle(NodeHandleType.ResizeMiddleLeft)
			Return
		End If

		Dim right As Rect = node.Bounds
		right.X = right.Right - 2
		right.Width = 2

		If right.Contains(e.MousePosition) Then
            e.HitResult = New NodeAdjustmentHandle(NodeHandleType.ResizeMiddleRight)
			Return
		End If

		Dim middle As Rect = node.Bounds
		middle.X = middle.X + middle.Width / 4
		middle.Width = middle.Width / 2

		If middle.Contains(e.MousePosition) Then
            e.HitResult = New NodeAdjustmentHandle(NodeHandleType.Move)
			Return
		End If

        e.HitResult = Nothing
	End Sub

	Private Sub diagram_NodeCreated(ByVal sender As Object, ByVal e As NodeEventArgs)
		e.Node.HandlesStyle = HandlesStyle.[Custom]
		e.Node.AnchorPattern = pattern

		' Place the box within the grid
		Dim bounds As Rect = e.Node.Bounds
		Dim topLeft As New Point(bounds.Left, bounds.Top)

		Dim grid As Grid = diagram.LaneGrid

		Dim cellBounds As Rect = Rect.Empty
		grid.GetCellFromPoint(topLeft, cellBounds)

		Dim pixel As Double = GetPixel()

		bounds.Y = cellBounds.Y + 4 * pixel
		bounds.Height = cellBounds.Height - 10 * pixel
		e.Node.Bounds = bounds
	End Sub

	Private Sub diagram_NodeCreating(ByVal sender As Object, ByVal e As NodeValidationEventArgs)
		Dim bounds As Rect = e.Node.Bounds
		Dim topLeft As New Point(bounds.Left, bounds.Top)

		Dim grid As Grid = diagram.LaneGrid

		Dim cellBounds As Rect = Rect.Empty
		Dim cell As ICell = grid.GetCellFromPoint(topLeft, cellBounds)

		If cell Is Nothing Then
			e.Cancel = True
		End If
	End Sub

	Private Sub diagram_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
		For Each node As ShapeNode In diagram.Nodes
			node.Stroke = Brushes.Black
		Next

		For Each link As DiagramLink In diagram.Links
			link.Stroke = Brushes.Black
			link.HeadStroke = Brushes.Black
			link.Brush = Brushes.Black
		Next

		For Each node As ShapeNode In diagram.Selection.Nodes
			node.Stroke = Brushes.Red
		Next

		For Each link As DiagramLink In diagram.Selection.Links
			link.Stroke = Brushes.Red
			link.HeadStroke = Brushes.Red
			link.Brush = Brushes.Red
		Next
	End Sub

	Private Sub diagram_NodeModifying(ByVal sender As Object, ByVal e As NodeValidationEventArgs)
		Dim bounds As Rect = e.Node.Bounds
		Dim topLeft As New Point(bounds.Left, bounds.Top + bounds.Height / 2)

		Dim grid As Grid = diagram.LaneGrid

		Dim cellBounds As Rect = Rect.Empty
		Dim cell As ICell = grid.GetCellFromPoint(topLeft, cellBounds)

        If cell Is Nothing Or cellBounds.IsEmpty Then
            e.Cancel = True
        End If
    End Sub

    Private Sub diagram_SelectionModifying(sender As Object, e As SelectionValidationEventArgs)

        Dim bounds = diagram.Selection.Bounds

        Dim header As Header = Nothing
        Dim gridBounds As Rect = diagram.LaneGrid.GetCellBounds(diagram.LaneGrid(header, header))
        If (Not gridBounds.Contains(bounds)) Then
            e.Cancel = True
        End If

    End Sub

    Private Sub diagram_LinkCreated(ByVal sender As Object, ByVal e As LinkEventArgs)
        e.Link.AllowMoveStart = False
        e.Link.AllowMoveEnd = False
        e.Link.HandlesStyle = HandlesStyle.[Custom]

        FixLink(e.Link)
    End Sub

    Private Sub diagram_NodeModified(ByVal sender As Object, ByVal e As NodeEventArgs)
        ' Place the box within the grid
        Dim bounds As Rect = e.Node.Bounds
        Dim topLeft As New Point(bounds.Left, bounds.Top + bounds.Height / 2)

        Dim grid As Grid = diagram.LaneGrid

        Dim cellBounds As Rect = Rect.Empty
        grid.GetCellFromPoint(topLeft, cellBounds)
        If (cellBounds.IsEmpty) Then
            Return
        End If

        Dim pixel As Double = GetPixel()

        bounds.Y = cellBounds.Y + 4 * pixel
        bounds.Height = cellBounds.Height - 10 * pixel
        e.Node.Bounds = bounds

        ' Fix links
        Dim links = e.Node.GetAllLinks()
        For Each link As DiagramLink In links
            FixLink(link)
        Next
    End Sub
End Class